#ifndef _MPL_CONCEPTS_FORWARD_SEQUENCE
#define _MPL_CONCEPTS_FORWARD_SEQUENCE

// MS compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
# pragma once
#endif

#include <boost/concept/assert.hpp>

#include "forward_iterator.hpp"
#include "integral_constant.hpp"
#include "boolean_integral_constant.hpp"

#include <boost/mpl/assert.hpp>
#include <boost/mpl/begin.hpp>
#include <boost/mpl/end.hpp>
#include <boost/mpl/size.hpp>
#include <boost/mpl/empty.hpp>
#include <boost/mpl/front.hpp>
#include <boost/mpl/void.hpp>
#include <boost/mpl/if.hpp>
#include <boost/mpl/equal_to.hpp>

#include <boost/type_traits/is_same.hpp>

namespace boost {
namespace mpl {

template<typename S>
struct ForwardSequence {
    typedef typename begin<S>::type t1;
    BOOST_CONCEPT_ASSERT(( ForwardIterator<t1> ));
    typedef typename end<S>::type t2;
    BOOST_CONCEPT_ASSERT(( ForwardIterator<t2> ));
    typedef typename size<S>::type t3;
    BOOST_CONCEPT_ASSERT(( IntegralConstant<t3> ));
    typedef typename empty<S>::type t4;
    BOOST_CONCEPT_ASSERT(( BooleanIntegralConstant<t4> ));
    // front is only defined if the sequence is not empty
    typedef typename if_<
        empty<S>,
        void_,
        front<S>
    >::type t5;
    // invariants
    // empty is true if and only if size is zero.
    BOOST_MPL_ASSERT((
        equal_to<
            typename empty<S>::type, 
            typename equal_to<size<S>, int_<0> >::type
        >
    ));
    // begin == end iff sequence is empty
    /* hmmm - traps in gcc 4.5.3 but not in microsoft
    */
    BOOST_MPL_ASSERT((
        equal_to<
            typename empty<S>::type,
            typename is_same<
                begin<S>, 
                end<S> 
            >::type
        >
    ));
};

} // mpl
} // boost

#endif // _MPL_CONCEPTS_FORWARD_SEQUENCE
