#ifndef _MPL_CONCEPTS_META_FUNCTION_CLASSES
#define _MPL_CONCEPTS_META_FUNCTION_CLASSES

// MS compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
# pragma once
#endif

#include <boost/concept/assert.hpp>
#include <boost/concept_check.hpp>
#include "boolean_integral_constant.hpp"

// Metafunction Class concepts
#include <boost/mpl/apply.hpp>

namespace boost {
namespace mpl {

template<typename Fun, template<class> class Ret >
struct NullaryMetaFunctionClass {
    BOOST_CONCEPT_USAGE(NullaryMetaFunctionClass)
    {
        typedef typename Fun:: template apply<>::type apply;
        BOOST_CONCEPT_ASSERT(( Ret<apply> ));
    }
};

template<typename Fun, template<class> class Ret, typename Arg>
struct UnaryMetaFunctionClass {
    BOOST_CONCEPT_USAGE(UnaryMetaFunctionClass)
    {
        typedef typename Fun:: template apply<Arg>::type type;
        BOOST_CONCEPT_ASSERT(( Ret<type> ));
    }
};

template<typename Fun, typename Arg>
struct UnaryMetaPredicateClass : 
    UnaryMetaFunctionClass<Fun, BooleanIntegralConstant, Arg>
{};

template<typename Fun, template<class> class Ret, typename Arg1, typename Arg2>
struct BinaryMetaFunctionClass {
    BOOST_CONCEPT_USAGE(BinaryMetaFunctionClass)
    {
        typedef typename Fun:: template apply<Arg1, Arg2>::type type;
        BOOST_CONCEPT_ASSERT(( Ret<type> ));
    }
};

template<typename Fun, typename Arg1, typename Arg2>
struct BinaryMetaPredicateClass : 
    BinaryMetaFunctionClass<Fun, BooleanIntegralConstant, Arg1, Arg2>
{};

} // mpl
} // boost

#endif // _MPL_CONCEPTS_META_FUNCTION_CLASSES
