#ifndef _MPL_CONCEPTS_META_FUNCTIONS
#define _MPL_CONCEPTS_META_FUNCTIONS

// MS compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
# pragma once
#endif

#include <boost/concept_check.hpp>
#include <boost/concept/assert.hpp>
#include "boolean_integral_constant.hpp"

namespace boost {
namespace mpl {

template<typename Fun>
struct NullaryMetaFunction {
    BOOST_CONCEPT_USAGE(NullaryMetaFunction)
    {
        typedef typename Fun::type type;
    }
};

template<template<typename> class Fun, typename Arg >
struct UnaryMetaFunction {
    BOOST_CONCEPT_USAGE(UnaryMetaFunction)
    {
        typedef typename Fun<Arg>::type type;
    }
};

template<template<typename> class Fun, typename Arg >
struct UnaryMetaPredicate { 
    BOOST_CONCEPT_USAGE(UnaryMetaPredicate)
    {
        typedef typename Fun<Arg>::type type;
        BOOST_CONCEPT_ASSERT(( BooleanIntegralConstant<type> ));
    }
};

template<template<typename, typename> class Fun, typename Arg1, typename Arg2 >
struct BinaryMetaFunction {
    BOOST_CONCEPT_USAGE(BinaryMetaFunction)
    {
        typedef typename Fun<Arg1, Arg2>::type type;
    }
};

template<template<typename, typename> class Fun, typename Arg1, typename Arg2 >
struct BinaryMetaPredicate
{
    BOOST_CONCEPT_USAGE(BinaryMetaPredicate)
    {
        typedef typename Fun<Arg1, Arg2>::type type;
        BOOST_CONCEPT_ASSERT(( BooleanIntegralConstant<type> ));
    }
};

} // mpl
} // boost

#endif // _MPL_CONCEPTS_META_FUNCTIONS
