#ifndef _MPL_CONCEPTS_RANDOM_ACCESS_ITERATOR
#define _MPL_CONCEPTS_RANDOM_ACCESS_ITERATOR

// MS compatible compilers support #pragma once
#if defined(_MSC_VER) && (_MSC_VER >= 1020)
# pragma once
#endif

#include <boost/concept/assert.hpp>
#include <boost/concept_check.hpp>

#include "integral_constant.hpp"
#include "bidirectional_iterator.hpp"

#include <boost/type_traits/is_convertible.hpp>

#include <boost/mpl/advance.hpp>
#include <boost/mpl/distance.hpp>
#include <boost/mpl/iterator_category.hpp>
#include <boost/mpl/iterator_tags.hpp>
#include <boost/mpl/int.hpp>

namespace boost {
namespace mpl {

template<typename I>
struct RandomAccessIterator {
    BOOST_CONCEPT_ASSERT(( BiDirectionalIterator<I> ));

    typedef typename prior<I>::type t2;
    BOOST_CONCEPT_ASSERT(( RandomAccessIterator<t2> ));
    typedef typename next<I>::type t3;
    BOOST_CONCEPT_ASSERT(( RandomAccessIterator<t3> ));
    BOOST_MPL_ASSERT(( 
        is_convertible< 
            typename I::category,
            random_access_iterator_tag
        >
    ));

    BOOST_CONCEPT_USAGE(RandomAccessIterator)
    {
        typedef typename advance<I, int_<0> >::type t4;
        BOOST_CONCEPT_ASSERT(( RandomAccessIterator<t4> ));
        typedef typename distance<I, I>::type t5;
        BOOST_CONCEPT_ASSERT(( IntegralConstant<t5> ));
        same_type(t5(), int_<0>());
    }
    template<typename T>
    void same_type(T &t1, T &t2);
};

} // mpl
} // boost

#endif // _MPL_CONCEPTS_RANDOM_ACCESS_ITERATOR
