///////////////////////////////////////////////////////////
// verify integral constant concepts
#include <boost/mpl/integral_c.hpp>

template <class T, T N>
struct integral_constant_archetype
{
    typedef boost::mpl::integral_c_tag tag;
    typedef T value_type;
    BOOST_STATIC_CONSTANT(T, value = N);
    typedef integral_constant_archetype type;
    operator value_type (){
        return value;
    }
    typedef integral_constant_archetype<T, (N+1)> next;
    typedef integral_constant_archetype<T, (N-1)> prior;
};

#include "concepts/integral_constant.hpp"

void test_integral_constant(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::IntegralConstant<
            integral_constant_archetype<int, 1>
        > 
    ));
}


template <bool B>
struct boolean_integral_constant_archetype
{
    typedef boost::mpl::integral_c_tag tag;
    typedef bool value_type;
    BOOST_STATIC_CONSTANT(bool, value = B);
    typedef boolean_integral_constant_archetype type;
    operator bool (){
        return value;
    }
};

#include "concepts/boolean_integral_constant.hpp"

void test_boolean_integral_constant(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::BooleanIntegralConstant<
            boolean_integral_constant_archetype<true>
        > 
    ));
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::BooleanIntegralConstant<
            boolean_integral_constant_archetype<false>
        > 
    ));
}

///////////////////////////////////////////////////////////
// verify forward iterator constant concepts

#include <boost/mpl/iterator_category.hpp>
#include <boost/mpl/iterator_tags.hpp>
#include <boost/mpl/deref.hpp>

struct forward_iterator_archetype {
    typedef boost::mpl::forward_iterator_tag category;
};

namespace boost {
namespace mpl {

template<>
struct deref<forward_iterator_archetype> {
    typedef int type;
};
template<>
struct next<forward_iterator_archetype> {
    typedef forward_iterator_archetype type;
};

} // mpl
} // boost

#include "concepts/forward_iterator.hpp"

void test_forward_iterator(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::ForwardIterator<
            forward_iterator_archetype
        >
    ));
}

///////////////////////////////////////////////////////////
// verify forward sequence constant concepts

#include <boost/mpl/begin.hpp>
#include <boost/mpl/end.hpp>
#include <boost/mpl/size.hpp>
#include <boost/mpl/empty.hpp>
#include <boost/mpl/front.hpp>

#include "forward_sequence_archetype.hpp"

struct forward_sequence_archetype {};

FORWARD_SEQUENCE_ARCHTYPE(forward_sequence_archetype)

//#include <boost/mpl/list.hpp>
#include "concepts/forward_sequence.hpp"

void test_forward_sequence(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::ForwardSequence<
            forward_sequence_archetype
        >
    ));
    //typedef boost::mpl::list<int, char> list_type;
}

///////////////////////////////////////////////////////////
// verify associative sequence constant concepts

#include <boost/mpl/has_key.hpp>
#include <boost/mpl/count.hpp>
#include <boost/mpl/order.hpp>
#include <boost/mpl/at.hpp>
#include <boost/mpl/key_type.hpp>
#include <boost/mpl/value_type.hpp>

struct associative_sequence_archetype {};

FORWARD_SEQUENCE_ARCHTYPE(associative_sequence_archetype)

namespace boost {
namespace mpl {

template<class K>
struct has_key<associative_sequence_archetype, K> {
    typedef boolean_integral_constant_archetype<true> type;
};

BOOST_CONCEPT_ASSERT(( integral_constant_archetype<unsigned int, 0> ));

template<class K>
struct count<associative_sequence_archetype, K> {
    typedef integral_constant_archetype<unsigned int, 0> type;
};

template<class K>
struct order<associative_sequence_archetype, K> {
    typedef integral_constant_archetype<unsigned int, 0> type;
};

template<class K>
struct at<associative_sequence_archetype, K> {
    typedef int type;
};

template<class X>
struct key_type<associative_sequence_archetype, X> {
    typedef X type;
};

template<class X>
struct value_type<associative_sequence_archetype, X> {
    typedef X type;
};
/**/

/*
    // this is in the docs, but I don't see it in the code !
    at<S, K,def>::type 	Any type 	Amortized constant time
*/

} // mpl
} // boost

#include "concepts/associative_sequence.hpp"

void test_associative_sequence(){
    typedef associative_sequence_archetype x;
    BOOST_CONCEPT_ASSERT(( boost::mpl::AssociativeSequence<x> ));
}

///////////////////////////////////////////////////////////
// verify metafunction concepts

#include "concepts/meta_functions.hpp"

struct nullary_metafunction_archetype {
    typedef int type;
};

void test_nullary_metafunction(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::NullaryMetaFunction<nullary_metafunction_archetype>
    ));
}

template<typename Arg>
struct unary_metafunction_archetype {
    typedef int type;
};

void test_unary_metafunction(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::UnaryMetaFunction<unary_metafunction_archetype, int>
    ));
}

template<typename Arg>
struct unary_meta_predicate_archetype {
    typedef boolean_integral_constant_archetype<true>::type type;
};


void test_unary_meta_predicate(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::UnaryMetaPredicate<unary_metafunction_archetype, int>
    ));
}

template<typename T1, typename T2>
struct binary_metafunction_archetype {
    typedef int type;
};

void test_binary_metafunction(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::BinaryMetaFunction<binary_metafunction_archetype, int, int>
    ));
}

template<typename T1, typename T2>
struct binary_meta_predicate_archetype {
    typedef boolean_integral_constant_archetype<true>::type type;
};

void test_binary_meta_predicate(){
    BOOST_CONCEPT_ASSERT(( 
        boost::mpl::BinaryMetaPredicate<binary_metafunction_archetype, int, int>
    ));
}

///////////////////////////////////////////////////////////
// verify metafunction class concepts

#include "concepts/meta_function_classes.hpp"
